@@ -89,49 +89,6 @@ static unsigned exam_expand_power()
 	return result;
 }
 
-static unsigned exam_sqrfree()
-{
-	unsigned result = 0;
-	symbol x("x"), y("y");
-	ex e1, e2;
-	
-	e1 = (1+x)*pow((2+x),2)*pow((3+x),3)*pow((4+x),4);
-	e2 = sqrfree(expand(e1),lst{x});
-	if (e1 != e2) {
-		clog << "sqrfree(expand(" << e1 << ")) erroneously returned "
-		     << e2 << endl;
-		++result;
-	}
-	
-	e1 = (x+y)*pow((x+2*y),2)*pow((x+3*y),3)*pow((x+4*y),4);
-	e2 = sqrfree(expand(e1));
-	if (e1 != e2) {
-		clog << "sqrfree(expand(" << e1 << ")) erroneously returned "
-		     << e2 << endl;
-		++result;
-	}
-	e2 = sqrfree(expand(e1),lst{x});
-	if (e1 != e2) {
-		clog << "sqrfree(expand(" << e1 << "),[x]) erroneously returned "
-		     << e2 << endl;
-		++result;
-	}
-	e2 = sqrfree(expand(e1),lst{y});
-	if (e1 != e2) {
-		clog << "sqrfree(expand(" << e1 << "),[y]) erroneously returned "
-		     << e2 << endl;
-		++result;
-	}
-	e2 = sqrfree(expand(e1),lst{x,y});
-	if (e1 != e2) {
-		clog << "sqrfree(expand(" << e1 << "),[x,y]) erroneously returned "
-		     << e2 << endl;
-		++result;
-	}
-	
-	return result;
-}
-
 /* Arithmetic Operators should behave just as one expects from built-in types.
  * When somebody screws up the operators this routine will most probably fail
  * to compile.  Unfortunately we can only test the stuff that is allowed, not
@@ -298,6 +255,29 @@ static unsigned exam_subs_algebraic()
 	return result;
 }
 
+/* Test suitable cases of the exponent power law: (e^t)^s=e^(ts). */
+static unsigned exam_exponent_power_law()
+{
+	unsigned result = 0;
+	symbol x("x");
+	realsymbol s("s");
+	possymbol t("t");
+
+	exmap pwr_exp =
+		{ {pow(exp(x), 2), exp(2*x)},
+		  {pow(exp(s), t), exp(s*t)},
+		  {exp(x)*pow(exp(x),-1), 1} };
+
+	for (auto e : pwr_exp) {
+		if (! (e.first.is_equal(e.second)) ) {
+			clog << "power of exponent  " << e.first << " produces error.\n";
+			++result;
+		}
+	}
+
+	return result;
+}
+
 unsigned exam_misc()
 {
 	unsigned result = 0;
@@ -307,11 +287,11 @@ unsigned exam_misc()
 	result += exam_expand_subs();  cout << '.' << flush;
 	result += exam_expand_subs2();  cout << '.' << flush;
 	result += exam_expand_power(); cout << '.' << flush;
-	result += exam_sqrfree(); cout << '.' << flush;
 	result += exam_operator_semantics(); cout << '.' << flush;
 	result += exam_subs(); cout << '.' << flush;
 	result += exam_joris(); cout << '.' << flush;
 	result += exam_subs_algebraic(); cout << '.' << flush;
+	result += exam_exponent_power_law(); cout << '.' << flush;
 	
 	return result;
 }
