@@ -3,7 +3,7 @@
  *  Some timings on series expansion of the Gamma function around a pole. */
 
 /*
- *  GiNaC Copyright (C) 1999-2014 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2020 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -55,15 +55,10 @@ unsigned time_gammaseries()
 
 	cout << "timing Laurent series expansion of Gamma function" << flush;
 
-	vector<unsigned> sizes;
+	vector<unsigned> sizes = {20, 25, 30, 35};
 	vector<double> times;
 	timer omega;
 
-	sizes.push_back(20);
-	sizes.push_back(25);
-	sizes.push_back(30);
-	sizes.push_back(35);
-
 	for (vector<unsigned>::iterator i=sizes.begin(); i!=sizes.end(); ++i) {
 		omega.start();
 		result += tgammaseries(*i);
