@@ -4,7 +4,7 @@
  *  Lewis and Michael Wester. */
 
 /*
- *  GiNaC Copyright (C) 1999-2016 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2020 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ using namespace GiNaC;
 #include <vector>
 using namespace std;
 
-static const bool do_test = true;  // set to true in order to run this beast
+const bool do_test = true;  // set to true in order to run this beast
 
 static unsigned test()
 {
