@@ -7,7 +7,7 @@
  */
 
 /*
- *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2020 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,13 +24,14 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <iostream>
-#include <vector>
 #include "ginac.h"
 #include "timer.h"
-using namespace std;
 using namespace GiNaC;
 
+#include <iostream>
+#include <vector>
+using namespace std;
+
 static unsigned vandermonde_det(unsigned size)
 {
 	unsigned result = 0;
@@ -57,7 +58,7 @@ static unsigned vandermonde_det(unsigned size)
 			vanddet *= M(i,1) - M(j,1);
 
 	if (expand(det - vanddet) != 0) {
-		clog << "Determaint of Vandermonde matrix " << endl
+		clog << "Determinant of Vandermonde matrix " << endl
 		     << "M==" << M << endl
 		     << "was miscalculated: det(M)==" << det << endl;
 		++result;
@@ -72,15 +73,10 @@ unsigned time_vandermonde()
 	
 	cout << "timing determinant of univariate symbolic Vandermonde matrices" << flush;
 	
-	vector<unsigned> sizes;
+	vector<unsigned> sizes = {8, 10, 12, 14};
 	vector<double> times;
 	timer swatch;
 	
-	sizes.push_back(8);
-	sizes.push_back(10);
-	sizes.push_back(12);
-	sizes.push_back(14);
-	
 	for (vector<unsigned>::iterator i=sizes.begin(); i!=sizes.end(); ++i) {
 		int count = 1;
 		swatch.start();
