@@ -3,7 +3,7 @@
  *  A simple stop watch class. */
 
 /*
- *  GiNaC Copyright (C) 1999-2015 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2020 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,7 +22,6 @@
 
 #ifdef HAVE_RUSAGE
 #include <sys/resource.h>
-#include <unistd.h>
 #include <sys/time.h>
 #else
 #include <ctime>
@@ -82,7 +81,6 @@ void timer::reset()
 
 double timer::read()
 {
-	double elapsed;
 #ifdef HAVE_RUSAGE
 	if (running())
 		getrusage(RUSAGE_SELF, &used2);
