@@ -24,9 +24,6 @@
 #include "registrar.h"
 #include "ex.h"
 #include "lst.h"
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 #include "version.h"
 
 #include <iostream>
