@@ -1797,8 +1797,7 @@ ex lcm(const ex &a, const ex &b, bool check_args)
  *  @return   vector of expairs (factor, exponent), sorted by exponents */
 static epvector sqrfree_yun(const ex &a, const symbol &x)
 {
-	// make polynomial primitive w.r.t. x
-	ex w = a.primpart(x);
+	ex w = a;
 	ex z = w.diff(x);
 	ex g = gcd(w, z);
 	if (g.is_zero()) {
@@ -1814,6 +1813,10 @@ static epvector sqrfree_yun(const ex &a, const symbol &x)
 	ex i = 0;  // exponent
 	do {
 		w = quo(w, g, x);
+		if (w.is_zero()) {
+			// hidden zero
+			break;
+		}
 		z = quo(z, g, x) - w.diff(x);
 		i += 1;
 		if (w.is_equal(x)) {
@@ -1829,20 +1832,18 @@ static epvector sqrfree_yun(const ex &a, const symbol &x)
 	} while (!z.is_zero());
 
 	// correct for lost factor
-	// (NB: This factor is not necessarily the unit and content lost when
-	// we made the polynomial primitive since Yun's algorithm only finds
-	// factors up to a unit.)
+	// (being based on GCDs, Yun's algorithm only finds factors up to a unit)
 	const ex lost_factor = quo(a, mul{factors}, x);
 	if (lost_factor.is_equal(_ex1)) {
 		// trivial lost factor
 		return factors;
 	}
-	if (factors[0].coeff.is_equal(1)) {
+	if (!factors.empty() && factors[0].coeff.is_equal(1)) {
 		// multiply factor^1 with lost_factor
 		factors[0].rest *= lost_factor;
 		return factors;
 	}
-	// only factors with higher exponent: prepend lost_factor^1 to the results
+	// no factor^1: prepend lost_factor^1 to the results
 	epvector results = {expair(lost_factor, 1)};
 	std::move(factors.begin(), factors.end(), std::back_inserter(results));
 	return results;
@@ -2019,8 +2020,36 @@ ex sqrfree_parfrac(const ex & a, const symbol & x)
 /** Create a symbol for replacing the expression "e" (or return a previously
  *  assigned symbol). The symbol and expression are appended to repl, for
  *  a later application of subs().
+ *  An entry in the replacement table repl can be changed in some cases.
+ *  If it was altered, we need to provide the modifier for the previously build expressions.
+ *  The modifier is an (ordered) list, because those substitutions need to be done in the
+ *  incremental order.
+ *  As an example let us consider a rationalisation of the expression
+ *      e = exp(2*x)*cos(exp(2*x)+1)*exp(x)
+ *  The first factor GiNaC denotes by something like symbol1 and will record:
+ *      e =symbol1*cos(symbol1 + 1)*exp(x)
+ *      repl = {symbol1 : exp(2*x)}
+ *  Similarly, the second factor would be denoted as symbol2 and we will have
+ *      e =symbol1*symbol2*exp(x)
+ *      repl = {symbol1 : exp(2*x), symbol2 : cos(symbol1 + 1)}
+ *  Denoting the third term as symbol3 GiNaC is willing to re-think exp(2*x) as
+ *  symbol3^2 rather than just symbol1. Here are two issues:
+ *  1) The replacement "symbol1 -> symbol3^2" in the previous part of the expression
+ *      needs to be done outside of the present routine;
+ *  2) The pair "symbol1 : exp(2*x)" shall be deleted from the replacement table repl.
+ *      However, this will create illegal substitution "symbol2 : cos(symbol1 + 1)" with
+ *      undefined symbol1.
+ *  These both problems are mitigated through the additions of the record
+ *  "symbol1==symbol3^2" to the list modifier. Changed length of the modifier signals
+ *  to the calling code that the previous portion of the expression needs to be
+ *  altered (it solves 1). Thus GiNaC can record now
+ *      e =symbol3^2*symbol2*symbol3
+ *      repl = {symbol2 : cos(symbol1 + 1), symbol3 : exp(x)}
+ *      modifier = {symbol1==symbol3^2}
+ *  Then, doing the backward substitutions the list modifier will be used to restore
+ *  such iterative substitutions in the right way (this solves 2).
  *  @see ex::normal */
-static ex replace_with_symbol(const ex & e, exmap & repl, exmap & rev_lookup)
+static ex replace_with_symbol(const ex & e, exmap & repl, exmap & rev_lookup, lst & modifier)
 {
 	// Since the repl contains replaced expressions we should search for them
 	ex e_replaced = e.subs(repl, subs_options::no_pattern);
@@ -2030,6 +2059,95 @@ static ex replace_with_symbol(const ex & e, exmap & repl, exmap & rev_lookup)
 	if (it != rev_lookup.end())
 		return it->second;
 
+	// The expression can be the base of substituted power, which requires a more careful search
+	if (! is_a<numeric>(e_replaced))
+		for (auto & it : repl)
+			if (is_a<power>(it.second) && e_replaced.is_equal(it.second.op(0))) {
+				ex degree = pow(it.second.op(1), _ex_1);
+				if (is_a<numeric>(degree) && ex_to<numeric>(degree).is_integer())
+					return pow(it.first, degree);
+			}
+
+	// We treat powers and the exponent functions differently because
+	// they can be rationalised more efficiently
+	if (is_a<function>(e_replaced) && is_ex_the_function(e_replaced, exp)) {
+		for (auto & it : repl) {
+			if (is_a<function>(it.second) && is_ex_the_function(e_replaced, exp)) {
+				ex ratio = normal(e_replaced.op(0) / it.second.op(0));
+				if (is_a<numeric>(ratio) && ex_to<numeric>(ratio).is_rational()) {
+					// Different exponents can be treated as powers of the same basic equation
+					if (ex_to<numeric>(ratio).is_integer()) {
+						// If ratio is an integer then this is simply the power of the existing symbol.
+						// std::clog << e_replaced << " is a " << ratio << " power of " << it.first << std::endl;
+						return dynallocate<power>(it.first, ratio);
+					} else {
+						// otherwise we need to give the replacement pattern to change
+						// the previous expression...
+						ex es = dynallocate<symbol>();
+						ex Num = numer(ratio);
+						modifier.append(it.first == power(es, denom(ratio)));
+						// std::clog << e_replaced << " is power " << Num << " and "
+						//		  << it.first << " is power " << denom(ratio) << " of the common base "
+						//		  << exp(e_replaced.op(0)/Num) << std::endl;
+						// ... and  modify the replacement tables
+						rev_lookup.erase(it.second);
+						rev_lookup.insert({exp(e_replaced.op(0)/Num), es});
+						repl.erase(it.first);
+						repl.insert({es, exp(e_replaced.op(0)/Num)});
+						return dynallocate<power>(es, Num);
+					}
+				}
+			}
+		}
+	} else if (is_a<power>(e_replaced) && !is_a<numeric>(e_replaced.op(0)) // We do not replace simple monomials like x^3 or sqrt(2)
+	           && ! (is_a<symbol>(e_replaced.op(0))
+	               && is_a<numeric>(e_replaced.op(1)) && ex_to<numeric>(e_replaced.op(1)).is_integer())) {
+		for (auto & it : repl) {
+			if (e_replaced.op(0).is_equal(it.second) // The base is an allocated symbol or base of power
+			    || (is_a<power>(it.second) && e_replaced.op(0).is_equal(it.second.op(0)))) {
+				ex ratio; // We bind together two above cases
+				if (is_a<power>(it.second))
+					ratio = normal(e_replaced.op(1) / it.second.op(1));
+				else
+					ratio = e_replaced.op(1);
+				if (is_a<numeric>(ratio) && ex_to<numeric>(ratio).is_rational())  {
+					// Different powers can be treated as powers of the same basic equation
+					if (ex_to<numeric>(ratio).is_integer()) {
+						// If ratio is an integer then this is simply the power of the existing symbol.
+						//std::clog << e_replaced << " is a " << ratio << " power of " << it.first << std::endl;
+						return dynallocate<power>(it.first, ratio);
+					} else {
+						// otherwise we need to give the replacement pattern to change
+						// the previous expression...
+						ex es = dynallocate<symbol>();
+						ex Num = numer(ratio);
+						modifier.append(it.first == power(es, denom(ratio)));
+						//std::clog << e_replaced << " is power " << Num << " and "
+						//		  << it.first << " is power " << denom(ratio) << " of the common base "
+						//		  << pow(e_replaced.op(0), e_replaced.op(1)/Num) << std::endl;
+						// ... and  modify the replacement tables
+						rev_lookup.erase(it.second);
+						rev_lookup.insert({pow(e_replaced.op(0), e_replaced.op(1)/Num), es});
+						repl.erase(it.first);
+						repl.insert({es, pow(e_replaced.op(0), e_replaced.op(1)/Num)});
+						return dynallocate<power>(es, Num);
+					}
+				}
+			}
+		}
+		// There is no existing substitution, thus we are creating a new one.
+		// This needs to be done separately to treat possible occurrences of
+		// b = e_replaced.op(0) elsewhere in the expression as pow(b, 1).
+		ex degree = pow(e_replaced.op(1), _ex_1);
+		if (is_a<numeric>(degree) && ex_to<numeric>(degree).is_integer()) {
+			ex es = dynallocate<symbol>();
+			modifier.append(e_replaced.op(0) == power(es, degree));
+			repl.insert({es, e_replaced});
+			rev_lookup.insert({e_replaced, es});
+			return es;
+		}
+	}
+
 	// Otherwise create new symbol and add to list, taking care that the
 	// replacement expression doesn't itself contain symbols from repl,
 	// because subs() is not recursive
@@ -2071,19 +2189,27 @@ struct normal_map_function : public map_function {
 /** Default implementation of ex::normal(). It normalizes the children and
  *  replaces the object with a temporary symbol.
  *  @see ex::normal */
-ex basic::normal(exmap & repl, exmap & rev_lookup) const
+ex basic::normal(exmap & repl, exmap & rev_lookup, lst & modifier) const
 {
 	if (nops() == 0)
-		return dynallocate<lst>({replace_with_symbol(*this, repl, rev_lookup), _ex1});
+		return dynallocate<lst>({replace_with_symbol(*this, repl, rev_lookup, modifier), _ex1});
 
 	normal_map_function map_normal;
-	return dynallocate<lst>({replace_with_symbol(map(map_normal), repl, rev_lookup), _ex1});
+	int nmod = modifier.nops(); // To watch new modifiers to the replacement list
+	lst result = dynallocate<lst>({replace_with_symbol(map(map_normal), repl, rev_lookup, modifier), _ex1});
+	for (int imod = nmod; imod < modifier.nops(); ++imod) {
+		exmap this_repl;
+		this_repl.insert(std::make_pair(modifier.op(imod).op(0), modifier.op(imod).op(1)));
+		result = ex_to<lst>(result.subs(this_repl, subs_options::no_pattern));
+	}
+
+	return result;
 }
 
 
 /** Implementation of ex::normal() for symbols. This returns the unmodified symbol.
  *  @see ex::normal */
-ex symbol::normal(exmap & repl, exmap & rev_lookup) const
+ex symbol::normal(exmap & repl, exmap & rev_lookup, lst & modifier) const
 {
 	return dynallocate<lst>({*this, _ex1});
 }
@@ -2093,19 +2219,19 @@ ex symbol::normal(exmap & repl, exmap & rev_lookup) const
  *  into re+I*im and replaces I and non-rational real numbers with a temporary
  *  symbol.
  *  @see ex::normal */
-ex numeric::normal(exmap & repl, exmap & rev_lookup) const
+ex numeric::normal(exmap & repl, exmap & rev_lookup, lst & modifier) const
 {
 	numeric num = numer();
 	ex numex = num;
 
 	if (num.is_real()) {
 		if (!num.is_integer())
-			numex = replace_with_symbol(numex, repl, rev_lookup);
+			numex = replace_with_symbol(numex, repl, rev_lookup, modifier);
 	} else { // complex
 		numeric re = num.real(), im = num.imag();
-		ex re_ex = re.is_rational() ? re : replace_with_symbol(re, repl, rev_lookup);
-		ex im_ex = im.is_rational() ? im : replace_with_symbol(im, repl, rev_lookup);
-		numex = re_ex + im_ex * replace_with_symbol(I, repl, rev_lookup);
+		ex re_ex = re.is_rational() ? re : replace_with_symbol(re, repl, rev_lookup, modifier);
+		ex im_ex = im.is_rational() ? im : replace_with_symbol(im, repl, rev_lookup, modifier);
+		numex = re_ex + im_ex * replace_with_symbol(I, repl, rev_lookup, modifier);
 	}
 
 	// Denominator is always a real integer (see numeric::denom())
@@ -2177,18 +2303,19 @@ static ex frac_cancel(const ex &n, const ex &d)
 /** Implementation of ex::normal() for a sum. It expands terms and performs
  *  fractional addition.
  *  @see ex::normal */
-ex add::normal(exmap & repl, exmap & rev_lookup) const
+ex add::normal(exmap & repl, exmap & rev_lookup, lst & modifier) const
 {
 	// Normalize children and split each one into numerator and denominator
 	exvector nums, dens;
 	nums.reserve(seq.size()+1);
 	dens.reserve(seq.size()+1);
+	int nmod = modifier.nops(); // To watch new modifiers to the replacement list
 	for (auto & it : seq) {
-		ex n = ex_to<basic>(recombine_pair_to_ex(it)).normal(repl, rev_lookup);
+		ex n = ex_to<basic>(recombine_pair_to_ex(it)).normal(repl, rev_lookup, modifier);
 		nums.push_back(n.op(0));
 		dens.push_back(n.op(1));
 	}
-	ex n = ex_to<numeric>(overall_coeff).normal(repl, rev_lookup);
+	ex n = ex_to<numeric>(overall_coeff).normal(repl, rev_lookup, modifier);
 	nums.push_back(n.op(0));
 	dens.push_back(n.op(1));
 	GINAC_ASSERT(nums.size() == dens.size());
@@ -2201,6 +2328,18 @@ ex add::normal(exmap & repl, exmap & rev_lookup) const
 	auto num_it = nums.begin(), num_itend = nums.end();
 	auto den_it = dens.begin(), den_itend = dens.end();
 //std::clog << " num = " << *num_it << ", den = " << *den_it << std::endl;
+	for (int imod = nmod; imod < modifier.nops(); ++imod) {
+		while (num_it != num_itend) {
+			*num_it = num_it->subs(modifier.op(imod), subs_options::no_pattern);
+			++num_it;
+			*den_it = den_it->subs(modifier.op(imod), subs_options::no_pattern);
+			++den_it;
+		}
+		// Reset iterators for the next round
+		num_it = nums.begin();
+		den_it = dens.begin();
+	}
+
 	ex num = *num_it++, den = *den_it++;
 	while (num_it != num_itend) {
 //std::clog << " num = " << *num_it << ", den = " << *den_it << std::endl;
@@ -2229,35 +2368,55 @@ ex add::normal(exmap & repl, exmap & rev_lookup) const
 /** Implementation of ex::normal() for a product. It cancels common factors
  *  from fractions.
  *  @see ex::normal() */
-ex mul::normal(exmap & repl, exmap & rev_lookup) const
+ex mul::normal(exmap & repl, exmap & rev_lookup, lst & modifier) const
 {
 	// Normalize children, separate into numerator and denominator
 	exvector num; num.reserve(seq.size());
 	exvector den; den.reserve(seq.size());
 	ex n;
+	int nmod = modifier.nops(); // To watch new modifiers to the replacement list
 	for (auto & it : seq) {
-		n = ex_to<basic>(recombine_pair_to_ex(it)).normal(repl, rev_lookup);
+		n = ex_to<basic>(recombine_pair_to_ex(it)).normal(repl, rev_lookup, modifier);
 		num.push_back(n.op(0));
 		den.push_back(n.op(1));
 	}
-	n = ex_to<numeric>(overall_coeff).normal(repl, rev_lookup);
+	n = ex_to<numeric>(overall_coeff).normal(repl, rev_lookup, modifier);
 	num.push_back(n.op(0));
 	den.push_back(n.op(1));
+	auto num_it = num.begin(), num_itend = num.end();
+	auto den_it = den.begin(), den_itend = den.end();
+	for (int imod = nmod; imod < modifier.nops(); ++imod) {
+		while (num_it != num_itend) {
+			*num_it = num_it->subs(modifier.op(imod), subs_options::no_pattern);
+			++num_it;
+			*den_it = den_it->subs(modifier.op(imod), subs_options::no_pattern);
+			++den_it;
+		}
+		num_it = num.begin();
+		den_it = den.begin();
+	}
 
 	// Perform fraction cancellation
 	return frac_cancel(dynallocate<mul>(num), dynallocate<mul>(den));
 }
 
 
-/** Implementation of ex::normal([B) for powers. It normalizes the basis,
+/** Implementation of ex::normal() for powers. It normalizes the basis,
  *  distributes integer exponents to numerator and denominator, and replaces
  *  non-integer powers by temporary symbols.
  *  @see ex::normal */
-ex power::normal(exmap & repl, exmap & rev_lookup) const
+ex power::normal(exmap & repl, exmap & rev_lookup, lst & modifier) const
 {
 	// Normalize basis and exponent (exponent gets reassembled)
-	ex n_basis = ex_to<basic>(basis).normal(repl, rev_lookup);
-	ex n_exponent = ex_to<basic>(exponent).normal(repl, rev_lookup);
+	int nmod = modifier.nops(); // To watch new modifiers to the replacement list
+	ex n_basis = ex_to<basic>(basis).normal(repl, rev_lookup, modifier);
+	for (int imod = nmod; imod < modifier.nops(); ++imod)
+		n_basis = n_basis.subs(modifier.op(imod), subs_options::no_pattern);
+
+	nmod = modifier.nops();
+	ex n_exponent = ex_to<basic>(exponent).normal(repl, rev_lookup, modifier);
+	for (int imod = nmod; imod < modifier.nops(); ++imod)
+		n_exponent = n_exponent.subs(modifier.op(imod), subs_options::no_pattern);
 	n_exponent = n_exponent.op(0) / n_exponent.op(1);
 
 	if (n_exponent.info(info_flags::integer)) {
@@ -2278,32 +2437,32 @@ ex power::normal(exmap & repl, exmap & rev_lookup) const
 		if (n_exponent.info(info_flags::positive)) {
 
 			// (a/b)^x -> {sym((a/b)^x), 1}
-			return dynallocate<lst>({replace_with_symbol(pow(n_basis.op(0) / n_basis.op(1), n_exponent), repl, rev_lookup), _ex1});
+			return dynallocate<lst>({replace_with_symbol(pow(n_basis.op(0) / n_basis.op(1), n_exponent), repl, rev_lookup, modifier), _ex1});
 
 		} else if (n_exponent.info(info_flags::negative)) {
 
 			if (n_basis.op(1).is_equal(_ex1)) {
 
 				// a^-x -> {1, sym(a^x)}
-				return dynallocate<lst>({_ex1, replace_with_symbol(pow(n_basis.op(0), -n_exponent), repl, rev_lookup)});
+				return dynallocate<lst>({_ex1, replace_with_symbol(pow(n_basis.op(0), -n_exponent), repl, rev_lookup, modifier)});
 
 			} else {
 
 				// (a/b)^-x -> {sym((b/a)^x), 1}
-				return dynallocate<lst>({replace_with_symbol(pow(n_basis.op(1) / n_basis.op(0), -n_exponent), repl, rev_lookup), _ex1});
+				return dynallocate<lst>({replace_with_symbol(pow(n_basis.op(1) / n_basis.op(0), -n_exponent), repl, rev_lookup, modifier), _ex1});
 			}
 		}
 	}
 
 	// (a/b)^x -> {sym((a/b)^x, 1}
-	return dynallocate<lst>({replace_with_symbol(pow(n_basis.op(0) / n_basis.op(1), n_exponent), repl, rev_lookup), _ex1});
+	return dynallocate<lst>({replace_with_symbol(pow(n_basis.op(0) / n_basis.op(1), n_exponent), repl, rev_lookup, modifier), _ex1});
 }
 
 
 /** Implementation of ex::normal() for pseries. It normalizes each coefficient
  *  and replaces the series by a temporary symbol.
  *  @see ex::normal */
-ex pseries::normal(exmap & repl, exmap & rev_lookup) const
+ex pseries::normal(exmap & repl, exmap & rev_lookup, lst & modifier) const
 {
 	epvector newseq;
 	for (auto & it : seq) {
@@ -2312,7 +2471,7 @@ ex pseries::normal(exmap & repl, exmap & rev_lookup) const
 			newseq.push_back(expair(restexp, it.coeff));
 	}
 	ex n = pseries(relational(var,point), std::move(newseq));
-	return dynallocate<lst>({replace_with_symbol(n, repl, rev_lookup), _ex1});
+	return dynallocate<lst>({replace_with_symbol(n, repl, rev_lookup, modifier), _ex1});
 }
 
 
@@ -2330,13 +2489,17 @@ ex pseries::normal(exmap & repl, exmap & rev_lookup) const
 ex ex::normal() const
 {
 	exmap repl, rev_lookup;
+	lst modifier;
 
-	ex e = bp->normal(repl, rev_lookup);
+	ex e = bp->normal(repl, rev_lookup, modifier);
 	GINAC_ASSERT(is_a<lst>(e));
 
 	// Re-insert replaced symbols
-	if (!repl.empty())
+	if (!repl.empty()) {
+		for(int i=0; i < modifier.nops(); ++i)
+			e = e.subs(modifier.op(i), subs_options::no_pattern);
 		e = e.subs(repl, subs_options::no_pattern);
+	}
 
 	// Convert {numerator, denominator} form back to fraction
 	return e.op(0) / e.op(1);
@@ -2351,15 +2514,20 @@ ex ex::normal() const
 ex ex::numer() const
 {
 	exmap repl, rev_lookup;
+	lst modifier;
 
-	ex e = bp->normal(repl, rev_lookup);
+	ex e = bp->normal(repl, rev_lookup, modifier);
 	GINAC_ASSERT(is_a<lst>(e));
 
 	// Re-insert replaced symbols
 	if (repl.empty())
 		return e.op(0);
-	else
+	else {
+		for(int i=0; i < modifier.nops(); ++i)
+			e = e.subs(modifier.op(i), subs_options::no_pattern);
+
 		return e.op(0).subs(repl, subs_options::no_pattern);
+	}
 }
 
 /** Get denominator of an expression. If the expression is not of the normal
@@ -2371,15 +2539,20 @@ ex ex::numer() const
 ex ex::denom() const
 {
 	exmap repl, rev_lookup;
+	lst modifier;
 
-	ex e = bp->normal(repl, rev_lookup);
+	ex e = bp->normal(repl, rev_lookup, modifier);
 	GINAC_ASSERT(is_a<lst>(e));
 
 	// Re-insert replaced symbols
 	if (repl.empty())
 		return e.op(1);
-	else
+	else {
+		for(int i=0; i < modifier.nops(); ++i)
+			e = e.subs(modifier.op(i), subs_options::no_pattern);
+
 		return e.op(1).subs(repl, subs_options::no_pattern);
+	}
 }
 
 /** Get numerator and denominator of an expression. If the expression is not
@@ -2391,15 +2564,20 @@ ex ex::denom() const
 ex ex::numer_denom() const
 {
 	exmap repl, rev_lookup;
+	lst modifier;
 
-	ex e = bp->normal(repl, rev_lookup);
+	ex e = bp->normal(repl, rev_lookup, modifier);
 	GINAC_ASSERT(is_a<lst>(e));
 
 	// Re-insert replaced symbols
 	if (repl.empty())
 		return e;
-	else
+	else {
+		for(int i=0; i < modifier.nops(); ++i)
+			e = e.subs(modifier.op(i), subs_options::no_pattern);
+
 		return e.subs(repl, subs_options::no_pattern);
+	}
 }
 
 
