@@ -3,7 +3,7 @@
  *  Factory for prime numbers. */
 
 /*
- *  GiNaC Copyright (C) 1999-2016 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2020 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -47,7 +47,7 @@ private:
 	// or even numeric. Eventually this will be fixed).
 	cln::cl_I last;
 	// This ensures coefficients are immediate.
-	static const int immediate_bits = 8*sizeof(void *) - __alignof__(void *);
+	static const int immediate_bits = 8*sizeof(void *) - alignof(void *);
 	static const long opt_hint = (1L << (immediate_bits >> 1)) - 1;
 public:
 	primes_factory()
