@@ -4,7 +4,7 @@
  *  of any interest to the user of the library. */
 
 /*
- *  GiNaC Copyright (C) 1999-2016 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2020 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,14 +25,9 @@
 #define GINAC_UTILS_H
 
 #include "assertion.h"
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include <functional>
-#ifdef HAVE_STDINT_H
-#include <stdint.h> // for uintptr_t
-#endif
+#include <cstdint> // for uintptr_t
 #include <string>
 
 namespace GiNaC {
@@ -69,37 +64,10 @@ inline int compare_pointers(const T * a, const T * b)
 	return 0;
 }
 
-#ifdef HAVE_STDINT_H
-typedef uintptr_t p_int;
-#else
-typedef unsigned long p_int;
-#endif
-
 /** Truncated multiplication with golden ratio, for computing hash values. */
-inline unsigned golden_ratio_hash(p_int n)
+inline unsigned golden_ratio_hash(uintptr_t n)
 {
-	// This function works much better when fast arithmetic with at
-	// least 64 significant bits is available.
-	if (sizeof(long) >= 8) {
-	// So 'long' has 64 bits.  Excellent!  We prefer it because it might be
-	// more efficient than 'long long'.
-	unsigned long l = n * 0x4f1bbcddUL;
-	return (unsigned)l;
-	}
-#ifdef HAVE_LONG_LONG
-	else if (sizeof(long long) >= 8) {
-	// This requires 'long long' (or an equivalent 64 bit type)---which is,
-	// unfortunately, not ANSI-C++-compliant.
-	// (Yet C99 demands it, which is reason for hope.)
-	unsigned long long l = n * 0x4f1bbcddULL;
-	return (unsigned)l;
-	}
-#endif
-	// Without a type with 64 significant bits do the multiplication manually
-	// by splitting n up into the lower and upper two bytes.
-	const unsigned n0 = (n & 0x0000ffffU);
-	const unsigned n1 = (n & 0xffff0000U) >> 16;
-	return (n0 * 0x0000bcddU) + ((n1 * 0x0000bcddU + n0 * 0x00004f1bU) << 16);
+	return n * UINT64_C(0x4f1bbcdd);
 }
 
 /* Compute the sign of a permutation of a container, with and without an
@@ -108,6 +76,7 @@ inline unsigned golden_ratio_hash(p_int n)
 template <class It>
 int permutation_sign(It first, It last)
 {
+	using std::swap;
 	if (first == last)
 		return 0;
 	--last;
@@ -122,7 +91,7 @@ int permutation_sign(It first, It last)
 		bool swapped = false;
 		while (i != first) {
 			if (*i < *other) {
-				std::iter_swap(other, i);
+				swap(*other, *i);
 				flag = other;
 				swapped = true;
 				sign = -sign;
@@ -143,7 +112,7 @@ int permutation_sign(It first, It last)
 		swapped = false;
 		while (i != last) {
 			if (*other < *i) {
-				std::iter_swap(i, other);
+				swap(*i, *other);
 				flag = other;
 				swapped = true;
 				sign = -sign;
@@ -303,6 +272,225 @@ again:
 	}
 }
 
+/** Base class for generating all bounded combinatorial partitions of an integer
+ *  n with exactly m parts in non-decreasing order.
+ */
+class basic_partition_generator {
+protected:
+	// Partitions n into m parts, not including zero parts.
+	// (Cf. OEIS sequence A008284; implementation adapted from Jörg Arndt's
+	// FXT library)
+	struct mpartition2
+	{
+		// partition: x[1] + x[2] + ... + x[m] = n and sentinel x[0] == 0
+		std::vector<unsigned> x;
+		unsigned n;   // n>0
+		unsigned m;   // 0<m<=n
+		mpartition2(unsigned n_, unsigned m_)
+		  : x(m_+1), n(n_), m(m_)
+		{
+			for (unsigned k=1; k<m; ++k)
+				x[k] = 1;
+			x[m] = n - m + 1;
+		}
+		bool next_partition()
+		{
+			unsigned u = x[m];  // last element
+			unsigned k = m;
+			unsigned s = u;
+			while (--k) {
+				s += x[k];
+				if (x[k] + 2 <= u)
+					break;
+			}
+			if (k==0)
+				return false;  // current is last
+			unsigned f = x[k] + 1;
+			while (k < m) {
+				x[k] = f;
+				s -= f;
+				++k;
+			}
+			x[m] = s;
+			return true;
+		}
+	};
+	mpartition2 mpgen;
+	basic_partition_generator(unsigned n_, unsigned m_)
+	  : mpgen(n_, m_)
+	{ }
+};
+
+/** Generate all bounded combinatorial partitions of an integer n with exactly
+ *  m parts (including zero parts) in non-decreasing order.
+ */
+class partition_with_zero_parts_generator : public basic_partition_generator {
+private:
+	unsigned m;  // number of parts 0<m
+	mutable std::vector<unsigned> partition;  // current partition
+	mutable bool current_updated;  // whether partition vector has been updated
+public:
+	partition_with_zero_parts_generator(unsigned n_, unsigned m_)
+	  : basic_partition_generator(n_, 1), m(m_), partition(m_), current_updated(false)
+	{ }
+	// returns current partition in non-decreasing order, padded with zeros
+	const std::vector<unsigned>& get() const
+	{
+		if (!current_updated) {
+			for (unsigned i = 0; i < m - mpgen.m; ++i)
+				partition[i] = 0;  // pad with zeros
+
+			for (unsigned i = m - mpgen.m; i < m; ++i)
+				partition[i] = mpgen.x[i - m + mpgen.m + 1];
+
+			current_updated = true;
+		}
+		return partition;
+	}
+	bool next()
+	{
+		current_updated = false;
+		if (!mpgen.next_partition()) {
+			if (mpgen.m == m || mpgen.m == mpgen.n)
+				return false;  // current is last
+			// increment number of parts
+			mpgen = mpartition2(mpgen.n, mpgen.m + 1);
+		}
+		return true;
+	}
+};
+
+/** Generate all bounded combinatorial partitions of an integer n with exactly
+ *  m parts (not including zero parts) in non-decreasing order.
+ */
+class partition_generator : public basic_partition_generator {
+private:
+	mutable std::vector<unsigned> partition;  // current partition
+	mutable bool current_updated;  // whether partition vector has been updated
+public:
+	partition_generator(unsigned n_, unsigned m_)
+	  : basic_partition_generator(n_, m_), partition(m_), current_updated(false)
+	{ }
+	// returns current partition in non-decreasing order, padded with zeros
+	const std::vector<unsigned>& get() const
+	{
+		if (!current_updated) {
+			for (unsigned i = 0; i < mpgen.m; ++i)
+				partition[i] = mpgen.x[i + 1];
+
+			current_updated = true;
+		}
+		return partition;
+	}
+	bool next()
+	{
+		current_updated = false;
+		return mpgen.next_partition();
+	}
+};
+
+/** Generate all compositions of a partition of an integer n, starting with the
+ *  compositions which has non-decreasing order.
+ */
+class composition_generator {
+private:
+	// Generates all distinct permutations of a multiset.
+	// (Based on Aaron Williams' algorithm 1 from "Loopless Generation of
+	// Multiset Permutations using a Constant Number of Variables by Prefix
+	// Shifts." <http://webhome.csc.uvic.ca/~haron/CoolMulti.pdf>)
+	struct coolmulti {
+		// element of singly linked list
+		struct element {
+			unsigned value;
+			element* next;
+			element(unsigned val, element* n)
+			  : value(val), next(n) {}
+			~element()
+			{   // recurses down to the end of the singly linked list
+				delete next;
+			}
+		};
+		element *head, *i, *after_i;
+		// NB: Partition must be sorted in non-decreasing order.
+		explicit coolmulti(const std::vector<unsigned>& partition)
+		  : head(nullptr), i(nullptr), after_i(nullptr)
+		{
+			for (unsigned n = 0; n < partition.size(); ++n) {
+				head = new element(partition[n], head);
+				if (n <= 1)
+					i = head;
+			}
+			after_i = i->next;
+		}
+		~coolmulti()
+		{   // deletes singly linked list
+			delete head;
+		}
+		void next_permutation()
+		{
+			element *before_k;
+			if (after_i->next != nullptr && i->value >= after_i->next->value)
+				before_k = after_i;
+			else
+				before_k = i;
+			element *k = before_k->next;
+			before_k->next = k->next;
+			k->next = head;
+			if (k->value < head->value)
+				i = k;
+			after_i = i->next;
+			head = k;
+		}
+		bool finished() const
+		{
+			return after_i->next == nullptr && after_i->value >= head->value;
+		}
+	} cmgen;
+	bool atend;  // needed for simplifying iteration over permutations
+	bool trivial;  // likewise, true if all elements are equal
+	mutable std::vector<unsigned> composition;  // current compositions
+	mutable bool current_updated;  // whether composition vector has been updated
+public:
+	explicit composition_generator(const std::vector<unsigned>& partition)
+	  : cmgen(partition), atend(false), trivial(true), composition(partition.size()), current_updated(false)
+	{
+		for (unsigned i=1; i<partition.size(); ++i)
+			trivial = trivial && (partition[0] == partition[i]);
+	}
+	const std::vector<unsigned>& get() const
+	{
+		if (!current_updated) {
+			coolmulti::element* it = cmgen.head;
+			size_t i = 0;
+			while (it != nullptr) {
+				composition[i] = it->value;
+				it = it->next;
+				++i;
+			}
+			current_updated = true;
+		}
+		return composition;
+	}
+	bool next()
+	{
+		// This ugly contortion is needed because the original coolmulti
+		// algorithm requires code duplication of the payload procedure,
+		// one before the loop and one inside it.
+		if (trivial || atend)
+			return false;
+		cmgen.next_permutation();
+		current_updated = false;
+		atend = cmgen.finished();
+		return true;
+	}
+};
+
+/** Compute the multinomial coefficient n!/(p1!*p2!*...*pk!) where
+ *  n = p1+p2+...+pk, i.e. p is a partition of n.
+ */
+const numeric
+multinomial_coefficient(const std::vector<unsigned> & p);
+
 
 // Collection of `construct on first use' wrappers for safely avoiding
 // internal object replication without running into the `static
